#! /usr/bin/perl 
#====================================================================
# What's this ?
#====================================================================
# This plugin check the state of each CPU of Compaq/HP servers.
# The server need Compaq/HP SNMP agent.
#
# Copyright (C) 2005 Cedric Temple
# Copyright (C) 2005 LINAGORA
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# GPL Licence : http://www.gnu.org/licenses/gpl.txt
#
#====================================================================
# More contributions on http://www.linagora.org
#====================================================================

#====================================================================
# Changelog
#====================================================================
#
# Version 0.1 (2005/02):
# - First implementation
# Author: Cedric Temple (LINAGORA)
#====================================================================


use lib "/usr/lib/nagios/plugins/compaq" ;
use utils qw($TIMEOUT %ERRORS &print_revision &support);
use vars qw($PROGNAME);
use Net::SNMP;
use Getopt::Long;
use vars qw($opt_V $opt_h $opt_v $opt_C $opt_H $opt_n);

$PROGNAME = $0;
sub print_help ();
sub print_usage ();

# get options
Getopt::Long::Configure('bundling');
GetOptions
    ("h"   => \$opt_h, "help"         => \$opt_h,
     "V"   => \$opt_V, "version"      => \$opt_V,
     "n=s" => \$opt_n, "number=s"     => \$opt_n,
     "v=s" => \$opt_v, "snmp=s"       => \$opt_v,
     "C=s" => \$opt_C, "community=s"  => \$opt_C,
     "H=s" => \$opt_H, "hostname=s"   => \$opt_H);

# version of plugin
if ($opt_V) {
        print_revision($PROGNAME,'$Revision: 1.0 $');
        exit $ERRORS{'OK'};
}

# help required
if ($opt_h) {
        print_help();
        exit $ERRORS{'OK'};
}

my $DIMMEventID = '.1.3.6.1.4.1.232.1.2.2.1.1.6';
# my $DIMMSpeed = '.1.3.6.1.4.1.11.2.23.21.3.1.1.4';

# get SNMP version
($opt_v) || ($opt_v = shift) || ($opt_v = "2c");
my $snmp = $1 if ($opt_v =~ /([-.A-Za-z0-9]+)/);
if ($snmp =~ /^2$/){
    $snmp = "2c";
}

# get SNMP community
($opt_C) || ($opt_C = shift) || ($opt_C = "public");
my $community = $1 if ($opt_C =~ /([-.A-Za-z0-9]+)/);


# SNMP Session
my ( $session, $error ) = Net::SNMP->session(-hostname  => $opt_H,-community => $community, -version  => $snmp);
if ( !defined($session) ) {
    print("UNKNOWN: $error");
    exit $ERRORS{'UNKNOWN'};
}


my $un;
$CPUnb = 0;



# tests if CPU is present or not
$response = undef;
if (!defined ($response = $session->get_table($DIMMEventID))) {
    $answer=$session->error;
    $session->close;
    $state = 'CRITICAL';
    printf ("$state: SNMP error with snmp version $snmp ($answer)\n");
    $session->close;
    exit $ERRORS{$state};
}
       
foreach $key ( keys %{$response}) {
    # 1 --> Unknown
    # 2 --> OK
    # 3 --> Degraded
    # 4 --> failed
    if ( $response->{$key} != 2 ) {
        print "CRITICAL: CPU $key is not presents!";
        exit $ERRORS{'CRITICAL'};
    }
    else {
	$CPUnb ++;
    }
}

if ( $CPUnb != $opt_n ) {
    print "CRITICAL: number of CPU presents: $CPUnb (!= $opt_n)\n";
    exit $ERRORS{'CRITICAL'};
}

print "OK: No error detected\n";
exit $ERRORS{'OK'};


sub print_usage () {
    print "\nUsage:\n";
    print "$PROGNAME\n";
    print "   -H (--hostname)   Hostname to query - (required)\n";
    print "   -C (--community)  SNMP read community (defaults to public,\n";
    print "                     used with SNMP v1 and v2c\n";
    print "   -v (--snmp_version)  1 for SNMP v1 (default)\n";
    print "                        2 or 2c for SNMP v2c\n";
    print "   -n (--number)     The number of CPUs that should have\n";
    print "   -V (--version)    Plugin version\n";
    print "   -h (--help)       usage help\n";

}

sub print_help () {
    print "Copyright (c) 2005 LINAGORA SA\n";
    print "Bugs to http://www.linagora.com/\n";
    print "\n";
    print_usage();
    print "\n";
}
